package se.chalmers.tdv.ddmin.examples;

import java.util.Collection;
import java.util.LinkedList;

import se.chalmers.tdv.ddmin.DDMin;
import se.chalmers.tdv.ddmin.DDTest;

public class BuggyFail {

	public static void main(String[] arg) {
		DDMin.DEBUG = false;
		// Minimising a failed input:
		minimiseReport("a debugging test");
		// Minimised result not smallest, as can be seen by:
		minimiseReport("a debuggin test");
		// Why is that?
	}

	private static void minimiseReport(String string) {

		// The input is a string, sensible choice of circumstances is the
		// characters that the string consists of.

		// Test method creates a String out of the circumstances and checks
		// whether an exception is thrown, in which case the test fails.

		DDTest<Character> myTest = new DDTest<Character>() {

			public int test(Collection<Character> circumstances) {
				String input = convertToString(circumstances);
				try {
					Buggy b = new Buggy();
					b.run(input);
					if (DDMin.DEBUG)
						System.out.println("  test " + circumstances + " PASS");
					return DDMin.PASS;
				} catch (Exception e) {
					if (DDMin.DEBUG)
						System.out.println("  test " + circumstances + " FAIL");
					return DDMin.FAIL;
				}
			}
		};

		DDMin<Character> ddMin = new DDMin<Character>(myTest);
		Collection<Character> input = convertToCollection(string);
		Collection<Character> minimalCase = ddMin.ddmin(input);

		System.out.println("Minimalised version of '" + string + "' is '"
				+ convertToString(minimalCase) + "'.");

	}

	private static Collection<Character> convertToCollection(String string) {
		Collection<Character> res = new LinkedList<Character>();
		for (char c : string.toCharArray())
			res.add(c);
		return res;
	}

	private static String convertToString(Collection<Character> chars) {
		StringBuilder sb = new StringBuilder();
		for (Character c : chars) {
			sb.append(c);
		}
		return sb.toString();
	}
}
